#include "MathBox.h"
#include "MathBoxFilter.h"
#include "PluginHub.h"
#include "PluginDependency.h"
#include "PluginOptionList.h"
#include "PluginImageSettings.h"

#include <QDebug>
#include <QComboBox>
#include <QAbstractButton>
#include <QMessageBox>


extern "C" BIBBLE_API B5Plugin *b5plugin() { return new MathBoxPlugin; }


bool MathBoxPlugin::init(PluginHub *hub, int id, int groupId, const QString &)
{
	m_hub = hub;
	m_id = id;
	m_groupId = groupId;
	return true;
}

bool MathBoxPlugin::registerFilters()
{
	MathBoxFilter *MathBox = new MathBoxFilter(m_hub, m_groupId);
	// Load right before perfectly clear
	// if (m_hub->addFilter(MathBox, PluginHub::Tail, QString())) {
	if (m_hub->addFilter(MathBox, PluginHub::Before, "PerfectlyClear")) {
		return true;
	}
	qDebug() << "Failed to load the MathBox filter";
	return false;
}

bool MathBoxPlugin::registerOptions()
{
	bool	b = true;
	
	b &= m_hub->addBoolOption( FILTERNAME::ENABLED, "mboxenabled",  tr("Enable MathBox"), tr("Enable MathBox Plug-in"), MBOX_FILTERNAME, false, 0,0);
	b &= m_hub->addStringOption( FILTERNAME::RFIELD,  "mboxformula1", tr("Red Formula"), tr("Formula for the red channel"),MBOX_FILTERNAME, "r", 0,0);
	b &= m_hub->addStringOption( FILTERNAME::GFIELD,  "mboxformula2", tr("Green Formula"),  tr("Formula for the green channel"), MBOX_FILTERNAME, "g", 0,0);
	b &= m_hub->addStringOption( FILTERNAME::BFIELD,  "mboxformula3", tr("Blue Formula"),  tr("Formula for the blue channel"), MBOX_FILTERNAME, "b", 0,0);
	b &= m_hub->addIntOption( FILTERNAME::XFIELD, "MBx", tr("Variable X"),  tr("Variable x"), MBOX_FILTERNAME, 1, 0,0);
	b &= m_hub->addIntOption( FILTERNAME::YFIELD, "MBy", tr("Variable Y"),  tr("Variable y"), MBOX_FILTERNAME, 1, 0,0);
	b &= m_hub->addIntOption( FILTERNAME::ZFIELD, "MBz", tr("Variable Z"), tr("Variable z"), MBOX_FILTERNAME, 1, 0,0);

	if ( b == false )
		qDebug() << "MathBox: Unable to register options.";
	
	return b;
	
}

bool MathBoxPlugin::finish()
{

        connect(m_hub, SIGNAL(settingsChanged(const PluginImageSettings &, const PluginImageSettings &, int)), this, SLOT(handleSettingsChange(const PluginImageSettings &, const PluginImageSettings &, int)));
        connect(m_hub, SIGNAL(controlChanged(const QString &, int, int , const PluginImageSettings &, const PluginOptionList &, PluginOptionList &)), this, SLOT(handleControlChange(const QString &, int, int , const PluginImageSettings &, const PluginOptionList &, PluginOptionList &)));
        connect(m_hub, SIGNAL(hotnessChanged(const PluginImageSettings &)), this, SLOT(handleHotnessChanged(const PluginImageSettings &)));

	return true;
}

PluginDependency *MathBoxPlugin::createDependency(const QString &name)
{
	Q_UNUSED(name);
	return NULL;
}

QList<QString> MathBoxPlugin::toolFiles()
{
	return QList<QString>();
}

QList<QWidget*> MathBoxPlugin::toolWidgets()
{
	return QList<QWidget*>();
}

void MathBoxPlugin::toolWidgetCreated(QWidget *uiWidget)
{
	Q_UNUSED( uiWidget );
}

//========================================================================================================
void MathBoxPlugin::handleSettingsChange(const PluginImageSettings &options, const PluginImageSettings &changed, int currentLayer)
{

    Q_UNUSED(options);
    Q_UNUSED(changed);
    Q_UNUSED(currentLayer);
}

void MathBoxPlugin::handleControlChange(const QString &optionName, int groupId, int layer, 
                                        const PluginImageSettings &options, 
                                        const PluginOptionList &current, PluginOptionList &changes)
{

    Q_UNUSED(optionName);
    Q_UNUSED(groupId);
    Q_UNUSED(current);
    Q_UNUSED(changes);
    //   v-- You need to add this if you're going to use the options --v
    //   some changes do not have options - check for NULL
    //    qDebug()<<"handlecontrolchange_prenull";
    if (options.options(layer) == NULL) return;
    
    return;
    
    //bool okay;

    //qDebug()<<"handlecontrolchange";
    // 
    /*
        if(
            changes.contains(MathBoxFilter::H, m_groupId)
          ||changes.contains(MathBoxFilter::L, m_groupId)
          ||changes.contains(MathBoxFilter::Contrast, m_groupId)
          ||changes.contains(MathBoxFilter::Mid, m_groupId)
          ||changes.contains(MathBoxFilter::Sat, m_groupId)
          ||changes.contains(MathBoxFilter::ProtectHighlights, m_groupId)
          ||changes.contains(MathBoxFilter::C0L, m_groupId)
          ||changes.contains(MathBoxFilter::C1L, m_groupId)
          ||changes.contains(MathBoxFilter::C2L, m_groupId)
          ||changes.contains(MathBoxFilter::C3L, m_groupId)
          ||changes.contains(MathBoxFilter::C4L, m_groupId)
          ||changes.contains(MathBoxFilter::C5L, m_groupId)
        ){
        changes.setBool(MathBoxFilter::Enable, m_groupId, true);
        }
*/

}

//========================================================================================================
void MathBoxPlugin::handleHotnessChanged(const PluginImageSettings &options)
{

    Q_UNUSED(options);
}

void MathBoxPlugin::reset() {
return;

/*
    if (PluginOptionList *options = m_hub->beginSettingsChange("Reset MathBox")) {
                options->setInt(MathBoxFilter::H, m_groupId, DEFAULT_H);
                options->setInt(MathBoxFilter::L, m_groupId, DEFAULT_L);
                options->setInt(MathBoxFilter::Contrast, m_groupId, DEFAULT_CONTRAST);
                options->setInt(MathBoxFilter::Mid, m_groupId, DEFAULT_MID);
                options->setInt(MathBoxFilter::Sat, m_groupId, DEFAULT_SAT);
                options->setBool(MathBoxFilter::ProtectHighlights, m_groupId, DEFAULT_HIGHLIGHT);
                options->setInt(MathBoxFilter::C0L, m_groupId, DEFAULT_EQUALIZER);
                options->setInt(MathBoxFilter::C1L, m_groupId, DEFAULT_EQUALIZER);
                options->setInt(MathBoxFilter::C2L, m_groupId, DEFAULT_EQUALIZER);
                options->setInt(MathBoxFilter::C3L, m_groupId, DEFAULT_EQUALIZER);
                options->setInt(MathBoxFilter::C4L, m_groupId, DEFAULT_EQUALIZER);
                options->setInt(MathBoxFilter::C5L, m_groupId, DEFAULT_EQUALIZER);
                m_hub->endSettingChange();
        }
*/
}
