#include "MathBoxFilter.h"

#include "PluginPipeSettings.h"
#include "PluginTile.h"
#include "PluginImage.h"
#include "PluginImageSettings.h"
#include "PluginOptionList.h"
#include <sys/param.h>
#include <muParser.h>


#include <QDebug>

using namespace std;


bool FILTERNAME::needsToRunLayer(const PluginImageSettings &options, const PluginPipeSettings &settings, PluginOptionList &layerOptions) const
{
	Q_UNUSED( settings );
	Q_UNUSED( options );
	
	bool okay;
	bool val = layerOptions.getBool(FILTERNAME::ENABLED, m_groupId, okay);
	return val;
}

bool FILTERNAME::isSlow(const PluginImageSettings &options, const PluginPipeSettings &settings) const
{
	Q_UNUSED( options );
	Q_UNUSED( settings );
	return false;
}

bool FILTERNAME::isLargeRadius(const PluginImageSettings &options, const PluginPipeSettings &settings) const
{
	Q_UNUSED( options );
	Q_UNUSED( settings );
	return false;
}

bool FILTERNAME::isPixelSource(const PluginImageSettings &options, const PluginPipeSettings &settings) const
{
	Q_UNUSED( options );
	Q_UNUSED( settings );
	return false;
}

QTransform FILTERNAME::transform(const PluginImageSettings &options, const PluginPipeSettings &settings, 
         const QSize &startSize) const
{
	Q_UNUSED( options );
	Q_UNUSED( settings );
	Q_UNUSED( startSize );
	
	return QTransform();	//	We don't warp the image so return an identity transform.

}

QSize FILTERNAME::size(const PluginImageSettings &options, const PluginPipeSettings &settings, const QSize &startSize) const
{
	Q_UNUSED( options );
	Q_UNUSED( settings );
	
	return startSize;
}

QList<PluginDependency*> FILTERNAME::prerequisites(const PluginImageSettings &options, const PluginPipeSettings &settings, PluginTile &tile) const
{
	Q_UNUSED( options );
	Q_UNUSED( settings );
	Q_UNUSED( tile );
	
	return QList<PluginDependency*>();	//	Empty list - we don't depend on any other tiles.
}

inline void MBox_RGBtoHSV( double r, double g, double b,
                           double *h, double *s, double *v )
{
	double min, max, delta;
	min = MIN(MIN( r, g), b );
	max = MAX(MAX( r, g), b );
	*v = max;				// v
	delta = max - min;
	if( max != 0.0 )
		*s = delta / max;		// s
	else {
		// r = g = b = 0		// s = 0, v is undefined
		*s = 0.0;
		*h = 0.0;
		return;
	}
	if ( delta == 0 ) {
		*h = 0.0;
	} else if( r == max )
		*h = ( g - b ) / delta;		// between yellow & magenta
	else if( g == max )
		*h = 2 + ( b - r ) / delta;	// between cyan & yellow
	else
		*h = 4 + ( r - g ) / delta;	// between magenta & cyan
	*h *= 1.047197551196597746;
	if( *h < 0 )
		*h += 6.2831853071795864769;
}

// Handle ranges that cross the 0 to 360 transition
/*
BOOL inRange(double value, int hue, int range) {
	BOOL rc = false;
	double R1 = hue + 0.5f*range;
	double R2 = hue - 0.5f*range;
	
	if ( R1 <=360 && R2 >=0 ) {
		rc = ( value <= R1 ) && (value >= R2);
	} else {
		R1 = ( R1 > 360 ) ? R1 - 360 : R1;
		R2 = ( R2 < 0 ) ? R2 + 360 : R2;
		rc = ( value <= R1 ) || ( value >= R2 );
	}

	return rc;
} */


void FILTERNAME::runLayer(const PluginImageSettings &options, 
                                    const PluginPipeSettings &settings, 
                                    PluginTile &tile, 
                                    PluginOptionList &layerOptions, 
                                    int layerPos) const
{
	Q_UNUSED( options );
	Q_UNUSED( settings );
	Q_UNUSED( layerOptions );
	
	// qDebug() << "MathBox Enter";
	
	PluginImage *	pSrcImg = tile.image();
	PluginImage *	pDstImg;
	
	//	Layer 0 output should always go into tile.image(), otherwise output should go into tile.tmpImage()
	if ( layerPos == 0 ) {
		pDstImg = tile.image();
	} else {
		pDstImg = tile.tmpImage();
	}
	
	// If we don't have an image, we can't do anything
	if ( pSrcImg == 0 || pDstImg == 0 )
	{
	        qDebug() << "MBoxConvert() failed - NULL image";
		return;
	}


     	int rw = pSrcImg->rowWords();	//	number of unsigned shorts in a row 
	int pw = pSrcImg->planeWords();	//	increment to get to the next color component in the plane
	int np = pSrcImg->nextPixel();	//	increment for the next pixel within the same color plane - if the pointer is
					//	pointing at a green pixel this increment will point to the next green pixel

	//	Use paddedWidth & paddedHeight to get the padded width & height of the tile.
	//	We could also use image->paddedWidth() & image->paddedHeight().
	int tileWidth = settings.paddedWidth();        //Tile Dimensions
	int tileHeight = settings.paddedHeight();

        double xVar=0.0,yVar=0.0,zVar=0.0;

        // qDebug() << "Start MBoxConvert()";

        using namespace mu;
        Parser redParser,greenParser,blueParser;
        
      
        QString redOption,greenOption,blueOption; 
	int xOption, yOption, zOption;
	bool	bOk = true;
	

        redOption = layerOptions.getString(FILTERNAME::RFIELD,m_groupId, bOk );
        greenOption = layerOptions.getString(FILTERNAME::GFIELD,m_groupId, bOk );
        blueOption = layerOptions.getString(FILTERNAME::BFIELD,m_groupId, bOk );
        xOption = layerOptions.getInt(FILTERNAME::XFIELD,m_groupId, bOk );
        yOption = layerOptions.getInt(FILTERNAME::YFIELD,m_groupId, bOk );
        zOption = layerOptions.getInt(FILTERNAME::ZFIELD,m_groupId, bOk );
        
        qDebug() << "x: " , xOption;
        qDebug() << "y: " , yOption;
        qDebug() << "z: " , zOption;
        
  
        
        xVar=(double) xOption;
        yVar=(double) yOption;
        zVar=(double) zOption;

        bool syntaxChecking = true;
        QString ParseMsg;
        
        try {
            redParser.SetExpr(redOption.toStdString());
            (void) redParser.GetUsedVar();
        } catch  (Parser::exception_type &e) {
        qDebug() << "MathBox : Red" , e.GetMsg().c_str();
            syntaxChecking = false;
        }
         try {
            greenParser.SetExpr(greenOption.toStdString());
            (void) greenParser.GetUsedVar();
        } catch  (Parser::exception_type &e) {
            qDebug() << "MathBox : Green" , e.GetMsg().c_str();
            syntaxChecking = false;
        }
        try {
            blueParser.SetExpr(blueOption.toStdString());
            (void) blueParser.GetUsedVar();
        } catch  (Parser::exception_type &e) {
            qDebug() << "MathBox : Blue" , e.GetMsg().c_str() ;
            syntaxChecking = false;
        }
       
        
        if(!syntaxChecking) {
          return ;
        }

	// Convert each scan line including the padding to monochrome
	// May not be necessary to convert the pad, but any processing after us that dips into the pad
	// may cause color artifacts around the edges of the image
	unsigned short *pIn, *pOut;
	double r=0, g=0, b=0, hsvH=0, hsvS=0, hsvV=0, lum=0;
        double r2=0,g2=0,b2=0;

        redParser.DefineVar("r",&r);
        redParser.DefineVar("g",&g);
        redParser.DefineVar("b",&b);
        redParser.DefineVar("h",&hsvH);
        redParser.DefineVar("s",&hsvS);
        redParser.DefineVar("v",&hsvV);
        redParser.DefineVar("l",&lum);
        redParser.DefineVar("x",&xVar);
        redParser.DefineVar("y",&yVar);
        redParser.DefineVar("z",&zVar);
        greenParser.DefineVar("r",&r);
        greenParser.DefineVar("g",&g);
        greenParser.DefineVar("b",&b);
        greenParser.DefineVar("h",&hsvH);
        greenParser.DefineVar("s",&hsvS);
        greenParser.DefineVar("v",&hsvV);
        greenParser.DefineVar("l",&lum);
        greenParser.DefineVar("x",&xVar);
        greenParser.DefineVar("y",&yVar);
        greenParser.DefineVar("z",&zVar);
        blueParser.DefineVar("r",&r);
        blueParser.DefineVar("g",&g);
        blueParser.DefineVar("b",&b);
        blueParser.DefineVar("h",&hsvH);
        blueParser.DefineVar("s",&hsvS);
        blueParser.DefineVar("v",&hsvV);
        blueParser.DefineVar("l",&lum);
        blueParser.DefineVar("x",&xVar);
        blueParser.DefineVar("y",&yVar);
        blueParser.DefineVar("z",&zVar);
        
      
	for( int y = 0; y < tileHeight; y++ ) 
	{
	        pIn = pSrcImg->data16() + rw * y;
	        pOut = pDstImg->data16() + rw * y;
	     	for ( int x = 0; x < tileWidth ; x++ ) {
		
	   	      r = ((double)*(pIn))/65535.0; 
		      g = ((double)*(pIn + pw))/65535.0; 
		      b = ((double)*(pIn + 2*pw))/65535.0;
                      lum = 0.299*r + 0.587*g + 0.114*b;
                      MBox_RGBtoHSV(r,g,b,&hsvH,&hsvS,&hsvV);
		   
                      try {
                         r2 = redParser.Eval();
                      } catch  (Parser::exception_type &e) {
                         r2 = r;
                          qDebug() << "MB:GREEN:"  , e.GetMsg().c_str();
                         //BMSG(e.GetMsg().c_str());
                      }
                      try {
                         g2 = greenParser.Eval();
                      } catch  (Parser::exception_type &e) {
                         g2 = g;
                          qDebug() << "MB:GREEN:"  , e.GetMsg().c_str();
                         //BMSG(e.GetMsg().c_str());
                      }
                      try {
                         b2 = blueParser.Eval();
                      } catch  (Parser::exception_type &e) {
                         b2 = b;
                          qDebug() << "MB:BLUE:"  , e.GetMsg().c_str();
                        //BMSG(e.GetMsg().c_str());
                     }

                     r = r2*65535.0;
                     g = g2*65535.0;
                     b = b2*65535.0;
                     if(r<0.0) r = 0.0;
                       else if (r>65535.0) r = 65535.0;
                     if(g<0.0) g = 0.0;
                       else if (g>65535.0) g = 65535.0;
                     if(b<0.0) b = 0.0;
                       else if (b>65535.0) b = 65535.0;

  		     *(pOut) = (unsigned short)(r + 0.5);
		     *(pOut + pw) = (unsigned short)(g + 0.5);
		     *(pOut + 2*pw) = (unsigned short)(b + 0.5);
		
			//	move to the next pixel (in the color plane)
		   pOut += np;
		   pIn += np;
	
		}
		
	}
     //   bibbleData->m_bsf.m_free(redOption);
      //  bibbleData->m_bsf.m_free(greenOption);
     //   bibbleData->m_bsf.m_free(blueOption);
	return;
}

